// Final Project - gs256 yl365
#include <mega32.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <delay.h>
#include <assert.h>

#define t1 100
#define RBUF_SIZE 60
#define TBUF_SIZE 40

// I like these definitions
#define begin {
#define end   }

// Subroutines
void gets_int(void);     // starts getting a string from serial line
void puts_int(void);     // starts a send to serial line
void initialize(void);   // all the usual mcu stuff
void console(void);      // serial console command interpreter
void PrintPrompt(void);  // command prompt

/* Scanner Headers */
char ch;
int ind;
unsigned long val;
char var;
enum {plus,minus,times,divide,exponent,lparen,rparen,number,variable,eof,illegal};
enum {DIFF, EVAL} state;
int SGet(void);
void Number(void);
void Variable(void);
char GetC(void);

/* Tree Defs */
typedef struct NodeDesc *Node;
struct NodeDesc {
  char kind; // plus, minus, times, divide, number, variable
  long val; // number: value
  char var; // variable: character
  Node left, right; // plus, minus, times, divide: children
};

/* Parser Headers */
int psym;
int divzero, error, perror, merror;
char evalflag;
long evalnum;
char deriv_var;
Node Expr(void);
Node Term(void);
Node ExpTerm(void);
Node Factor(void);

/* Differentiation */
Node Diff(Node orig);
void Simplify(Node diff);

/* Node Functions */
void NodeFree(Node orig);
Node NodeCopy(Node orig);
int NodeCompare(Node a, Node b);
void PrintLinear(Node root);
void Print(Node root, long level);

// Variables
unsigned char led;
unsigned int time1;
long var_val[27];
char var_valid[27];

// RXC ISR variables
unsigned char r_index;             // current string index
unsigned char r_buffer[RBUF_SIZE]; // input string
unsigned char r_ready;             // flag for receive done
unsigned char r_char;              // current character

// TX empty ISR variables
unsigned char t_index;             // current string index
unsigned char t_buffer[TBUF_SIZE]; // output string
unsigned char t_ready;             // flag for transmit done
unsigned char t_char;              // current character

// Timer0 Compare ISR
interrupt [TIM0_COMP] void timer0_compare(void)
begin
  if (time1 > 0) --time1;
end

// UART character-ready ISR
interrupt [USART_RXC] void uart_rec(void)
begin
	r_char = UDR;    // get a char
	UDR = r_char;    // then print it
	//build the input string
	if (r_char != '\r')
	begin
	  if (r_char == 0x08) // if backspace detected
	    r_index--; // then move back one char in buffer
	  else
	    r_buffer[r_index++] = r_char;
	end
	else
	begin
		putchar('\n');              // use putchar to avoid overwrite
		while (r_index < RBUF_SIZE) // zero terminate
		begin
		  r_buffer[r_index++] = 0x00;
		end
		r_ready = 1;                // signal cmd processor
		UCSRB.7 = 0;                // kill receive ISR
	end
end

// UART xmit-empty ISR
interrupt [USART_DRE] void uart_send(void)
begin
  // Toggle LED
  led = led ^ 0xff;
  PORTD.7 = led;
  t_char = t_buffer[++t_index];
  if (t_char == 0)
  begin
		UCSRB.5 = 0;       // kill xmit-empty ISR
		t_ready = 1;       // transmit done
    // Turn off LED
    led = 0xff;
    PORTD.7 = led;
	end
	else
	  UDR = t_char ;     // send the char
end

// non-blocking keyboard check initializes ISR-driven
// receive. This routine merely sets up the ISR, which then
// does all the work of getting a command.
void gets_int(void)
begin
  r_ready=0;
  r_index=0;
  UCSRB.7=1;  // enable receive interrupt
end

// nonblocking print: initializes ISR-driven
// transmit. This routine merely sets up the ISR, then
// send one character, The ISR does all the work.
void puts_int(void)
begin
  t_ready=0;
  t_index=0;
  if (t_buffer[0] > 0)
  begin
  	putchar(t_buffer[0]); // Send first character
  	UCSRB.5 = 1;          // Turn on xmit-empty ISR
  end
end

void console(void)
begin
  Node root, diff;
  int i;

  time1 = t1;
  if (r_ready)
  begin
    // Init flags
    ind = 0;
    divzero = 0;
    error = 0;
    perror = 0;
    merror = 0;
    evalflag = 0;
    if((r_buffer[0] == 'd') && (r_buffer[1] == 'i') && (r_buffer[2] == 'f') &&
      (r_buffer[3] == 'f')) {
      state = DIFF;
      // Send command prompt
      PrintPrompt();
      // Ready to receive again
      gets_int();
      return;
    }
    if((r_buffer[0] == 'e') && (r_buffer[1] == 'v') && (r_buffer[2] == 'a') &&
      (r_buffer[3] == 'l')) {
      state = EVAL;
      // Send command prompt
      PrintPrompt();
      // Ready to receive again
      gets_int();
      return;
    }
    if((r_buffer[0] == 'p') && (r_buffer[1] == 'r') && (r_buffer[2] == 'i') &&
      (r_buffer[3] == 'n') && (r_buffer[4] == 't')) {
      printf("\r\n");
      for(i = 0; i < 26; i++)
      {
        if(var_valid[i] != 0)
          printf("%c = %ld  ", 'a' + i, var_val[i]);
      }
      if(var_valid[26] != 0)
        printf("ans = %ld", var_val[26]);
      printf("\r\n");
      // Send command prompt
      PrintPrompt();
      // Ready to receive again
      gets_int();
      return;     
    }
    // Get character
    ch = GetC();
    // Get symbol from scanner
    psym = SGet();
    // Get expr tree
    root = Expr();
    // Simplify expr tree
    Simplify(root);
    Simplify(root);
    // Check we reached EOF
    if(psym != eof) {
      perror = 1; // If not throw error
    }
    if(state == DIFF) {
      // Get variable to take deriv
      ch = GetC();
      psym = SGet();
      deriv_var = 'x';
      if (psym == variable) {
        deriv_var = var;
      } else if (psym == number) {
        evalnum = val;
        evalflag = 1;
      }
      ch = GetC();
      psym = SGet();
      if (psym == number) {
        evalnum = val;
        evalflag = 1;
      } else if (psym != eof) {
        perror = 1;
      }
      diff = Diff(root);
      Simplify(diff);
      Simplify(diff);
    }
                
    printf("\r\n");
    if (perror != 0) {
      printf("Syntax error. Type sw to change modes.\r\n");
    } else if (error != 0) {
      printf("Can't Diff Expr\r\n");
    } else if (divzero != 0) {
      printf("Div by zero\r\n");
    } else if (merror != 0) {
      printf("Out of Memory\r\n");
    } else {
      if(state == DIFF) {
        printf("df(%c)/d%c = ", deriv_var, deriv_var);
        PrintLinear(diff);
      } else {
        printf("ans = ");
        PrintLinear(root);
        if(root->kind == number) {
          var_valid[26] = 1;
          var_val[26] = root->val;
        } else {
          var_valid[26] = 0;
          var_val[26] = 0;
        }
      }
      printf("\r\n");
    }

    NodeFree(root);
    free(root);
    if(state == DIFF) {
      NodeFree(diff);
      free(diff);
    }

    // Send command prompt
    PrintPrompt();

    // Ready to receive again
    gets_int();
  end
end

// Command Prompt
void PrintPrompt(void)
begin
  while(t_ready == 0);
  if(state == DIFF)
    sprintf(t_buffer, "\r\ndiff# ");
  else
    sprintf(t_buffer, "\r\neval# ");
  puts_int();
end

// Entry point and task scheduler loop
void main(void)
begin
  initialize();
  // main task scheduler loop -- never exits!
  while(1)
  begin
    if (time1 == 0) console();
  end
end

// Setup
void initialize(void)
begin
  int i;
  
  // Set port as output
  DDRD = 0xff;

  // Turn off LED
  led = 0xff;
  PORTD.7 = led;

  // Serial setup
  UCSRB = 0x18;
  UBRRL = 103; // 9600 baud
  putsf("\r\nLoading CalcParser...\r\n");

  // Set up timer 0
  OCR0 = 249;           // 1 ms interrupts
  TIMSK = 2;            // Turn on timer0 cmp-match ISR
  TCCR0 = 0b00001011;   // prescalar to 64 and clr-on-match

  // Init transmit/receive flags
  r_ready = 0;
  t_ready = 1;
  
  // Init timer
  time1 = t1;
  
  // Init state;
  state = EVAL;
  
  for(i = 0; i < 27; i++) {
    var_val[i] = 0;
    var_valid[i] = 0;
  }
  
  //crank up the ISRs
  #asm
    sei
  #endasm

  // Ready to receive
  gets_int();

  // Print Command Prompt
  PrintPrompt();
end

/* Scanner Functions */
void Variable()
{
  var = ch;
  ch = GetC();
  if((ch >= 'a') && (ch <= 'z'))
    perror = 1;
}

void Number()
{
  val = 0;
  while(('0' <= ch) && (ch <= '9')) {
    val = val*10 + ch - '0';
    ch = GetC();
  }
}

int SGet()
{
  register int sym;
  while((ch != '\0') && (ch != ';') && (ch <= ' ')) {
    ch = GetC();
  }
  switch(ch) {
    case '\0': case ';':
      sym = eof;
      break;
    case '+':
      sym = plus;
      ch = GetC();
      break;
    case '-':
      sym = minus;
      ch = GetC();
      break;
    case '*':
      sym = times;
      ch = GetC();
      break;
    case '/':
      sym = divide;
      ch = GetC();
      break;
    case '(':
      sym = lparen;
      ch = GetC();
      break;
    case ')':
      sym = rparen;
      ch = GetC();
      break;
    case '^':
      sym = exponent;
      ch = GetC();
      break;
    case '0': case '1': case '2': case '3':
    case '4': case '5': case '6': case '7':
    case '8': case '9':
      sym = number;
      Number();
      break;
    case 'a': case 'b': case 'c': case 'd':
    case 'e': case 'f': case 'g': case 'h':
    case 'i': case 'j': case 'k': case 'l':
    case 'm': case 'n': case 'o': case 'p':
    case 'q': case 'r': case 's': case 't':
    case 'u': case 'v': case 'w': case 'x':
    case 'y': case 'z':
      sym = variable;
      Variable();
      break;
    default:
      sym = illegal;
  }
  return sym;
}

/* Parser Functions */
Node Factor()
{
  char neg;
  Node root;

  neg = 0;
  if(!((psym == number) || (psym == minus) || (psym == variable) || (psym == lparen))) {
    perror = 1;
    return NULL;
  }
  if(psym == minus) {
    psym = SGet();
    neg = 1;
  }
  if(psym == number) {
    root = malloc(sizeof(NodeDesc)); // dynamic memory allocation is implemented
    if(root == NULL) { // not enough memory
      merror = 1; // Out of Mem error
      return NULL;
    }
    root->kind = number;
    if(neg == 1) // assigns a negative value if a '-' sign is observed at the input
      root->val = 0 - val;
    else
      root->val = val;
    root->left = NULL;
    root->right = NULL;
    psym = SGet();
    return root;
  } else if(psym == variable) {
    root = malloc(sizeof(NodeDesc));
    if(root == NULL) {
      merror = 1;
      return NULL;
    }
    root->kind = times;// if the root node is a var input
    root->var = '\0';
    root->val = 0;
    root->left = malloc(sizeof(NodeDesc));
    if(root->left == NULL) {
      merror = 1;
      return NULL;
    }
    root->left->kind = variable;
    root->left->var = var;
    root->left->val = 0;
    root->left->left = NULL;
    root->left->right = NULL;
    root->right = malloc(sizeof(NodeDesc));
    if(root->right == NULL) {
      merror = 1;
      return NULL;
    }
    root->right->kind = number;
    root->right->var = '\0';
    if(neg == 1)
      root->right->val = -1;
    else
      root->right->val = 1;
    root->right->left = NULL;
    root->right->right = NULL;
    psym = SGet();
    return root;
  } else {
    root = malloc(sizeof(NodeDesc));
    if(root == NULL) {
      merror = 1;
      return NULL;
    }
    root->kind = times;
    psym = SGet();
    root->left = Expr();
    root->right = malloc(sizeof(NodeDesc));
    if(root->right == NULL) {
      merror = 1;
      return NULL;
    }
    root->right->kind = number;
    root->right->var = '\0';
    if(neg == 1)
      root->right->val = -1;
    else
      root->right->val = 1;
    root->right->left = NULL;
    root->right->right = NULL;
    if(psym != rparen) {
      perror = 1;
      return NULL;
    }
    psym = SGet();
    return(root);
  }
}

Node ExpTerm()
{
  Node ret, tmp;
  ret = Factor();
  while((psym == exponent)) {
    tmp = ret;
    ret = malloc(sizeof(NodeDesc));
    if(ret == NULL) {
      merror = 1;
      return NULL;
    }
    ret->kind = exponent;
    psym = SGet();
    ret->left = Factor();
    ret->right = tmp;
  }
  return ret;
}

Node Term()
{
  Node ret, tmp;
  ret = ExpTerm();
  while((psym == times)||(psym == divide)) {
    tmp = ret;
    ret = malloc(sizeof(NodeDesc));
    if(ret == NULL) {
      merror = 1;
      return NULL;
    }
    if(psym == times)
      ret->kind = times;
    else
      ret->kind = divide;
    psym = SGet();
    ret->left = ExpTerm();
    if((ret->left->kind == number) && (ret->kind == divide)) {
      if(ret->left->val == 0) {
				divzero = 1;
      }
    }
    ret->right = tmp;
  }
  return ret;
}

Node Expr()
{
  Node ret, tmp;
  ret = Term();
  while((psym == plus)||(psym == minus)) {
    tmp = ret;
    ret = malloc(sizeof(NodeDesc));
    if(ret == NULL) {
      merror = 1;
      return NULL;
    }
    if(psym == plus)
      ret->kind = plus;
    else
      ret->kind = minus;
    psym = SGet();
    ret->left = Term();
    ret->right = tmp;
  }
  return ret;
}

void Print(Node root, long level)
{
  long i;
  if (root != NULL) {
    Print(root->right, level+1);
    for (i = 0; i < level; i++) printf("  ");
    switch (root->kind) {
      case plus: printf("+\r\n"); break;
      case minus: printf("-\r\n"); break;
      case times: printf("*\r\n"); break;
      case divide: printf("/\r\n"); break;
      case exponent: printf("^\r\n"); break;
      case number:
        if (root->val < 0)
          printf("(%ld)", root->val);
        else
          printf("%ld", root->val);       
        break;
      case variable: printf("%c\r\n", root->var); break;
    }
    Print(root->left, level+1);
  }
}

void PrintLinear(Node root)
{
  if (root != NULL) {
    if(!((root->kind == number)||(root->kind == variable)))
      printf("(");
    PrintLinear(root->right);
    switch (root->kind) {
      case plus: printf("+"); break;
      case minus: printf("-"); break;
      case times: printf("*"); break;
      case divide: printf("/"); break;
      case exponent: printf("^"); break;
      case number:
        if (root->val < 0)
          printf("(%ld)", root->val);
        else
          printf("%ld", root->val);       
        break;
      case variable: printf("%c", root->var); break;
    }
    PrintLinear(root->left);
    if(!((root->kind == number)||(root->kind == variable)))
      printf(")");
  }
}

Node Diff(Node orig)
{
  Node diff;
  if((orig != NULL) && (merror == 0)) {
    if(orig->kind == plus) {
      diff = malloc(sizeof(NodeDesc));
      if(diff == NULL) {
        merror = 1;
        return NULL;
      }
      diff->kind = plus;
      diff->right = Diff(orig->right);
      diff->left = Diff(orig->left);
      Simplify(diff);
      return diff;
    } else if(orig->kind == minus) {
      diff = malloc(sizeof(NodeDesc));
      if(diff == NULL) {
        merror = 1;
        return NULL;
      }
      diff->kind = minus;
      diff->right = Diff(orig->right);
      diff->left = Diff(orig->left);
      Simplify(diff);
      return diff;
    } else if(orig->kind == times) {
      diff = malloc(sizeof(NodeDesc));
      if(diff == NULL) {
        merror = 1;
        return NULL;
      }
      diff->kind = plus;

      diff->right = malloc(sizeof(NodeDesc));
      if(diff->right == NULL) {
        merror = 1;
        return NULL;
      }
      diff->right->kind = times;
      diff->right->right = Diff(orig->right);
      diff->right->left = NodeCopy(orig->left);

      diff->left = malloc(sizeof(NodeDesc));
      if(diff->left == NULL) {
        merror = 1;
        return NULL;
      }
      diff->left->kind = times;
      diff->left->right = Diff(orig->left);
      diff->left->left = NodeCopy(orig->right);
      Simplify(diff);
      return diff;
    } else if(orig->kind == divide) {
      diff = malloc(sizeof(NodeDesc));
      if(diff == NULL) {
        merror = 1;
        return NULL;
      }
      diff->kind = divide;

      diff->right= malloc(sizeof(NodeDesc));
      if(diff->right == NULL) {
        merror = 1;
        return NULL;
      }
      diff->right->kind = minus;

      diff->right->right = malloc(sizeof(NodeDesc));
      if(diff->right->right == NULL) {
        merror = 1;
        return NULL;
      }
      diff->right->right->kind = times;
      diff->right->right->right = Diff(orig->right);
      diff->right->right->left = NodeCopy(orig->left);

      diff->right->left = malloc(sizeof(NodeDesc));
      if(diff->right->left == NULL) {
        merror = 1;
        return NULL;
      }
      if(diff == NULL) merror = 1;
      diff->right->left->kind = times;
      diff->right->left->right = Diff(orig->left);
      diff->right->left->left = NodeCopy(orig->right);

      diff->left = malloc(sizeof(NodeDesc));
      if(diff->left == NULL) {
        merror = 1;
        return NULL;
      }
      diff->left->kind = times;
      diff->left->right = NodeCopy(orig->left);
      diff->left->left = NodeCopy(orig->left);
      Simplify(diff);
      return diff;
    } else if(orig->kind == number) {
      diff = malloc(sizeof(NodeDesc));
      if(diff == NULL) {
        merror = 1;
        return NULL;
      }
      diff->kind = number;
      diff->val = 0;
      diff->left = NULL;
      diff->right = NULL;
      return diff;
    } else if(orig->kind == variable) {
      if(orig->var == deriv_var)
      {
        diff = malloc(sizeof(NodeDesc));
        if(diff == NULL) {
          merror = 1;
          return NULL;
        }
        diff->kind = number;
        diff->val = 1;
        diff->left = NULL;
        diff->right = NULL;
      } else {
        diff = malloc(sizeof(NodeDesc));
        if(diff == NULL) {
          merror = 1;
          return NULL;
        }
        diff->kind = number;
        diff->val = 0;
        diff->left = NULL;
        diff->right = NULL;
      }
      return diff;
    } else if(orig->kind == exponent) {
      diff = malloc(sizeof(NodeDesc));
      if(diff == NULL) {
        merror = 1;
        return NULL;
      }
      diff->kind = times;

      diff->right = malloc(sizeof(NodeDesc));
      if(diff->right == NULL) {
        merror = 1;
        return NULL;
      }
      diff->right->kind = times;
      diff->right->right = Diff(orig->right);
      diff->right->left = NodeCopy(orig->left);
      
      diff->left = malloc(sizeof(NodeDesc));
      if(diff->left == NULL) {
        merror = 1;
        return NULL;
      }
      diff->left->kind = exponent;
      diff->left->right = NodeCopy(orig->right);
      diff->left->left = NodeCopy(orig->left);
      Simplify(diff->left->left);
      if(diff->left->left->kind != number)
            error = 1;
      else
        diff->left->left->val = diff->left->left->val - 1;
      Simplify(diff);
      return diff;      
    }
  }
  return NULL;
}

/* Recursive function to create a new simplified tree in memory from
   differentiated tree and return a pointer (does not destroy the diff tree) */
void Simplify(Node diff)
{
  Node tmp;
  if((diff != NULL) && (merror == 0))
  {
    /* Recursive simplify left and right nodes */
    Simplify(diff->right);
    Simplify(diff->left);

    if((diff->kind == variable) && (diff->var == deriv_var) && (evalflag == 1)) {
      diff->kind = number;
      diff->val = evalnum;
      diff->var = '\0';
    }

    if((diff->left != NULL) && (diff->right != NULL)) {
      if(diff->kind == plus)
      {
        /* Once done, simplify based on already simplified left/right branches */
        if((diff->left->kind == number) && (diff->left->val == 0)) {
          /* f + 0 = f */
          tmp = diff->right;
          free(diff->left);
          diff->kind = diff->right->kind;
          diff->val = diff->right->val;
          diff->var = diff->right->var;
          diff->left = diff->right->left;
          diff->right = diff->right->right;
          free(tmp);
          return;
        } else if((diff->right->kind == number) && (diff->right->val == 0)) {
          /* 0 + f = f */
          tmp = diff->left;
          free(diff->right);
          diff->kind = diff->left->kind;
          diff->val = diff->left->val;
          diff->var = diff->left->var;
          diff->right = diff->left->right;
          diff->left = diff->left->left;
          free(tmp);
          return;
        } else if(NodeCompare(diff->right, diff->left) == 0) {
          /* f + f = 2*f */
          diff->kind = times;
          NodeFree(diff->left);
          diff->left->kind = number;
          diff->left->val = 2;
          diff->left->var = '\0';
          diff->left->right = NULL;
          diff->left->left = NULL;
          return;
        } else if((diff->right->kind == number) && (diff->left->kind == number)) {
          diff->kind = number;
          diff->val = diff->right->val + diff->left->val;
          diff->var = '\0';
          free(diff->left);
          free(diff->right);
          diff->left = NULL;
          diff->right = NULL;
          return;
        }
      } else if(diff->kind == minus) {
        if(NodeCompare(diff->right, diff->left) == 0) {
          /* f - f = 0 */
          diff->kind = number;
          diff->val = 0;
          diff->var = '\0';
          NodeFree(diff->left);
          NodeFree(diff->right);
          free(diff->left);
          free(diff->right);
          diff->left = NULL;
          diff->right = NULL;
          return;
        } else if((diff->right->kind == number) && (diff->left->kind == number)) {
          diff->kind = number;
          diff->val = diff->right->val - diff->left->val;
          diff->var = '\0';
          free(diff->left);
          free(diff->right);
          diff->left = NULL;
          diff->right = NULL;
          return;
        }
      } else if(diff->kind == divide) {
        if((diff->right->kind == number) && (diff->left->kind == number)) {
          diff->kind = number;
          diff->val = diff->right->val / diff->left->val;
          diff->var = '\0';
          free(diff->left);
          free(diff->right);
          diff->left = NULL;
          diff->right = NULL;
          return;         
        }
      } else if(diff->kind == times) {
        if(((diff->left->kind == number) && (diff->left->val == 0)) ||
            ((diff->right->kind == number) && (diff->right->val == 0)))
        {
          diff->kind = number;
          diff->val = 0;
          diff->var = '\0';
          NodeFree(diff->left);
          NodeFree(diff->right);
          free(diff->left);
          free(diff->right);
          diff->left = NULL;
          diff->right = NULL;
          return;
        } else if((diff->left->kind == number) && (diff->left->val == 1)) {
          tmp = diff->right;
          free(diff->left);
          diff->kind = diff->right->kind;
          diff->val = diff->right->val;
          diff->var = diff->right->var;
          diff->left = diff->right->left;
          diff->right = diff->right->right;
          free(tmp);
          return;
        } else if((diff->right->kind == number) && (diff->right->val == 1)) {
          tmp = diff->left;
          free(diff->right);
          diff->kind = diff->left->kind;
          diff->val = diff->left->val;
          diff->var = diff->left->var;
          diff->right = diff->left->right;
          diff->left = diff->left->left;
          free(tmp);
          return;
        } else if((diff->right->kind == number) && (diff->left->kind == number)) {
          diff->kind = number;
          diff->val = diff->right->val * diff->left->val;
          diff->var = '\0';
          free(diff->left);
          free(diff->right);
          diff->left = NULL;
          diff->right = NULL;
          return;
        }
      } else if(diff->kind == exponent) {
        if((diff->left->kind == number) && (diff->left->val == 0)) {
          NodeFree(diff->right);
          free(diff->left);
          free(diff->right);
          diff->left = NULL;
          diff->right = NULL;
          diff->val = 1;
          diff->kind = number;
          diff->var = '\0';
          return;
        } else if((diff->left->val == 1) && (diff->left->kind == number)) {
          tmp = diff->right;
          free(diff->left);
          diff->val = diff->right->val;
          diff->kind = diff->right->kind;
          diff->var = diff->right->var;
          diff->left = diff->right->left;
          diff->right = diff->right->right;
          free(tmp);
          return;
        } else if((diff->left->kind == number) && (diff->right->kind == number)) {
          diff->kind = number;
          diff->val = 1;
          while(diff->left->val > 0) {
            diff->val = diff->val * diff->right->val;
            diff->left->val = diff->left->val - 1;
          }
          if(diff->left->val < 0)
            diff->val = 0;
          diff->var = '\0';
          free(diff->left);
          free(diff->right);
          diff->left = NULL;
          diff->right = NULL;
        }
      }
    }
  }
}

Node NodeCopy(Node orig)
{
  Node copy;
  if((orig != NULL) && (merror == 0))
  {
    copy = malloc(sizeof(NodeDesc));
    if(copy == NULL) {
      merror = 1;
      return NULL;
    }
    copy->kind = orig->kind;
    copy->var = orig->var;
    copy->val = orig->val;
    copy->right = NodeCopy(orig->right);
    copy->left = NodeCopy(orig->left);
    return copy;
  }
  return NULL;
}

void NodeFree(Node orig)
{
  if(orig != NULL)
  {
    NodeFree(orig->right);
    NodeFree(orig->left);
    free(orig->right);
    free(orig->left);
    orig->right = NULL;
    orig->left = NULL;
  }
}

/* Return value < 0 means subnodes A and B are not equal */
int NodeCompare(Node a, Node b)
{
  int ret = 0;
  if((a != NULL) && (b != NULL)) {
    ret = ret + NodeCompare(a->right, b->right);
    ret = ret + NodeCompare(a->left, b->left);
    if(a->kind != b->kind)
      return (ret - 1);
    else if((a->kind == number) && (b->kind == number))
    {
      if(a->val != b->val)
        return (ret - 1);
    } else if((a->kind == variable) && (b->kind == variable)) {
      if(a->var != b->var)
        return (ret - 1);
    } else {
      return ret;
    }
  }
  return ret;
}

char GetC(void)
{
  if(ind == RBUF_SIZE) {
    return('\0');
  } else {
    return(r_buffer[ind++]);
  }
}